"""DO NOT MODIFY. Auto-generated by script/fingerprint_frontend."""

FINGERPRINTS = {
    "compatibility.js": "8e4c44b5f4288cc48ec1ba94a9bec812",
    "core.js": "d4a7cb8c80c62b536764e0e81385f6aa",
    "frontend.html": "7d599996578579600f1000d6d25e649d",
    "mdi.html": "e91f61a039ed0a9936e7ee5360da3870",
    "micromarkdown-js.html": "93b5ec4016f0bba585521cf4d18dec1a",
    "panels/ha-panel-automation.html": "1982116c49ad26ee8d89295edc797084",
    "panels/ha-panel-config.html": "fafeac72f83dd6cc42218f8978f6a7af",
    "panels/ha-panel-dev-event.html": "77784d5f0c73fcc3b29b6cc050bdf324",
    "panels/ha-panel-dev-info.html": "24e888ec7a8acd0c395b34396e9001bc",
    "panels/ha-panel-dev-service.html": "86a42a17f4894478b6b77bc636beafd0",
    "panels/ha-panel-dev-state.html": "31ef6ffe3347cdda5bb0cbbc54b62cde",
    "panels/ha-panel-dev-template.html": "d1d76e20fe9622cddee33e67318abde8",
    "panels/ha-panel-hassio.html": "262d31efd9add719e0325da5cf79a096",
    "panels/ha-panel-history.html": "35177e2046c9a4191c8f51f8160255ce",
    "panels/ha-panel-iframe.html": "238189f21e670b6dcfac937e5ebd7d3b",
    "panels/ha-panel-kiosk.html": "2ac2df41bd447600692a0054892fc094",
    "panels/ha-panel-logbook.html": "7c45bd41c146ec38b9938b8a5188bb0d",
    "panels/ha-panel-map.html": "50501cd53eb4304e9e46eb719aa894b7",
    "panels/ha-panel-shopping-list.html": "1d7126efc9ff9a102df7465d803a11d1",
    "panels/ha-panel-zwave.html": "422f95f820f8b6b231265351ffcf4dd1"
}
